/*
   CLOCKBOY by florian anwander, 06/2023.
   This software for Arduino Nano is designed for the Eurorackmodule CLOCKBOY
   It converts DIN-Sync or MIDI-Clock into "analogue" Triggersignals
   Details on https://florian-anwander.de/clockboy

   1.19 WaitforPause / Pause / WaitForPlay are blinking
   1.21 VMC-Bus Clock
   1.22 Blinking while edit changed
   1.23 Blinking while edit changed back once again
   1.24 MIDI-Thru off when MIDI-Clock as source (Thru of clock caused double time MIDI-Out)
   1.25 minor corrections on pause function for MIDI and DIN
   1.26 Divisor rate switches only on beat-one, minor bugfixes and improvements
*/
#include <MIDI.h>
#include <EEPROM.h>
MIDI_CREATE_DEFAULT_INSTANCE();

/* Buttons */
#define startPin    12
#define stopPin     11

/* Sockets 0-Version
  #define outGatePin   3
  #define outStartPin  4
  #define outStopPin   5
  #define outResetPin  6
  #define outBeat1Pin  7
  #define outClockPin  2
  #define dinClockPin  8
  #define dinGatePin   9

  #define vmcClockPin  A6
  #define vmcGatePin   A7


  #define ledBeat1Pin  7
  #define ledResetPin  6
  #define ledStopPin   5
  #define ledStartPin  4
  #define ledGatePin   3
  #define ledClockPin  2
  /* */
/* Sockets 1-Version */
#define outGatePin   5
#define outStartPin  6
#define outStopPin   7
#define outResetPin  8
#define outBeat1Pin  9
#define outClockPin  4
#define dinClockPin  2
#define dinGatePin   3

#define vmcClockPin  A6
#define vmcGatePin   A7

#define ledBeat1Pin  A2
#define ledResetPin  A3
#define ledStopPin   A4
#define ledStartPin  A5
#define ledGatePin   A0
#define ledClockPin  A1
/* */

/*Variables for the reading logic of inputs and buttons */
boolean startState = 0;
boolean stopState = 0;
boolean startStatePrev = 0;
boolean stopStatePrev = 0;

boolean dinClockState = 0;
boolean dinGateState = 0;
boolean dinClockStatePrev = 0;
boolean dinGateStatePrev = 0;
boolean clockState = 0;
boolean clockStatePrev = 0;
boolean resetState = 0;
boolean doReset = 0;

/* Counters and Flags*/
uint16_t barClockCount = 0;
uint16_t barCount;
uint16_t clockCount = 0;
uint16_t clockLength ;
uint8_t ledBlink8th = 0;
uint16_t clockTickCount = 0;
boolean sendClockTickOn = 0;
boolean sendClockTickOff = 0;
uint16_t startCount = 0;
uint8_t sendStartTrigger = 0;
uint16_t stopCount = 0;
uint8_t sendStopTrigger = 0;
uint16_t resetCount = 0;
uint8_t sendResetTrigger = 0;
uint16_t beat1Count = 0;
uint8_t sendBeat1Trigger = 0;
uint8_t sendBeat1TriggerDone = 0;

uint16_t shuffleLength = 0;
boolean shuffleStep = 1;
uint16_t shuffleCount = 0;

uint8_t sendDinClock1 = 0;
uint8_t sendDinClock2 = 0;
uint8_t dinCount = 0;
uint8_t din1Count = 0;
uint8_t sendDinClock = 0;

uint8_t midiclockcounter = 0;
uint16_t clockStepCount = 0;
uint16_t cycleCount = 0;
uint16_t maxCycleCount = 0;

uint16_t ledblink = 0;      // tell leds to blink
uint8_t ledblinkcount = 0; // how often to blink
uint8_t ledblinkcounter = 0;   // helper variables
uint8_t ledblinkdirection = 1; // helper variables
uint8_t ledblinklength = 1; // for short or long blinks
String ledpattern = ""; // tells which leds to blink


int mult = 1;
uint16_t triggerLength = 500;
/*
  This paratmeter determines how long the start/stop/reset-triggers are.
  It does NOT affect the length of the clock-pulses!
*/

boolean playMode = 0;
/* 0=stopped, 1=running */

uint8_t pause = 0;
/* 0=normal running, 1=waitForHalt, 2=stopped, 3=waitForPlay */

uint8_t mode = 0;
/*
   mode=0 Runmode
   mode=1 Parameter selection mode
   mode=2 Value edit mode
*/

uint8_t setMidiChannel = 0;
uint8_t control76 = 0;
uint8_t channel;
/* These three are required for the set Channel function via CC76*/


int param;
/*
    0=Divider on ClockOut
    1=Shuffle
    2=Resetmode
    3=Barcount for the Beat-1 trigger
    4=Master = DIN / MIDI
    5=ContinousClock / GatedClock
    6=MIDI-Channel
*/

int value[6];
/*
    Divider(0) CLOCK will blink
     0=16         clock will be lit
     1=8          start will be lit
     2=4          beat1 will be lit
     3=32         beat1 will be lit
     4=48         stop  will be lit
     5=96         running will be lit
    shuffle(1) START will blink
     0=straight
     1=1
     2=2
     3=3
     4=4
     5=5
    RESETMODE(2) RESET will blink
     0=after Stop (A154/A157)           clock will be lit
     1=at Start (A155)                  start will be lit
     2=each last 16th (Vermona Slicer)  reset will be lit

    BEAT1-BARCOUNT(3) BEAT1 will blink
     0=1Bar  4/4  clock will be lit
     1=2Bars 4/4  start will be lit
     2=4Bars 4/4  reset will be lit
     3=1Bar  3/4  beat1 will be lit
     4=1Bar  5/4  stop  will be lit
     5=1Bars 7/8  running will be lit

    CLOCK SOURCE(4) STOP will blink
     0=DIN, analog paused                 clock will be lit
     1=DIN, analog and MIDI paused        start will be lit
     2=MIDI, analog paused                reset will be lit
     3=MIDI, analog, DIN paused           beat1 will be lit
     4=VMC, analog paused                 stop will be lit
     5=MIDI, analog, DIN and MIDI paused  running will be lit

    Gate / GatedClock(5) RUNNING will blink
     0=RunGate    clock will be lit
     1=GatedClock start will be lit

    MIDI
     1-16

*/
int maxvalue[7];

/* Variables for reading the order of button presses */
String keyPressOrder = "";
String keyPressOrderLast = "";
boolean keyPressOrderLastIsSet = 0;

boolean toggle = 0;
boolean debug = 0;

void setup() {
  MIDI.begin();

  if (EEPROM.read(0) > 5) {
    factoryreset();
  }

  value[0] = EEPROM.read(0);
  value[1] = EEPROM.read(1);
  value[2] = EEPROM.read(2);
  value[3] = EEPROM.read(3);
  value[4] = EEPROM.read(4);
  value[5] = EEPROM.read(5);
  value[6] = EEPROM.read(6);

  //SET DEFINDE VALUES FOR TESTING ONLY
  /*
    value[0] = 0; //div 1/16
    value[1] = 0; //shuffle
    value[2] = 0; //reset after stop
    value[3] = 0; //one bar beat1
    value[4] = 2; //MIDI-Master
    value[5] = 0; //Continuous Clock
    value[6] = 1; //MIDI
    // */

  channel = value[6];
  MIDI.setInputChannel(channel);

  setDivisor() ;

  /* the following can be achieved also
     in the variable definitions, but I like
     it more separated in one line for each
     parameter like this*/
  maxvalue[0] = 5;
  maxvalue[1] = 5;
  maxvalue[2] = 2;
  maxvalue[3] = 5;
  maxvalue[4] = 5;
  maxvalue[5] = 1;
  maxvalue[6] = 16;

  pinMode(startPin, INPUT_PULLUP);
  pinMode(stopPin, INPUT_PULLUP);

  pinMode(ledClockPin, OUTPUT);
  pinMode(ledStartPin , OUTPUT);
  pinMode(ledStopPin, OUTPUT);
  pinMode(ledResetPin, OUTPUT);
  pinMode(ledGatePin, OUTPUT);
  pinMode(ledBeat1Pin, OUTPUT);

  pinMode(outClockPin, OUTPUT);
  pinMode(outStartPin , OUTPUT);
  pinMode(outStopPin, OUTPUT);
  pinMode(outResetPin, OUTPUT);
  pinMode(outGatePin, OUTPUT);
  pinMode(outBeat1Pin, OUTPUT);

  pinMode(vmcClockPin, INPUT);
  pinMode(vmcGatePin, INPUT);

  if ( value[4] <= 1 ) {
    pinMode(dinClockPin, INPUT);
    pinMode(dinGatePin, INPUT);
  }
  if ( value[4] >= 2 ) {
    pinMode(dinClockPin, OUTPUT);
    pinMode(dinGatePin, OUTPUT);
  }

  MIDI.setThruFilterMode(0);
  if ( (value[4] == 2) || (value[4] == 3)) {
    MIDI.turnThruOn();
  } else {
    MIDI.turnThruOff();
  }
  sendCirclingLeds();
  mode = 0;
}

void loop() {


  /* Read the buttons */
  startState = !digitalRead(startPin);
  stopState = !digitalRead(stopPin);



  /* MIDI IS MASTER */
  if (value[4] == 2 || value[4] == 3) {
    if (MIDI.read()) {
      byte type = MIDI.getType();
      switch (type) {
        // THIS IS ONLY IF USED AS BREAKABLE CLOCK CHAIN
        //  case midi::NoteOff:
        //  if (MIDI.getData1() == 127 && MIDI.getData2() == 1 && MIDI.getChannel() == 1) {
        //    clockCount = 0;
        //    barClockCount = 0;
        //  }
        //  break;

        case midi::Start:
          playMode = 1;
          digitalWrite(dinGatePin, HIGH);
          if (mode == 0) {
            digitalWrite(outGatePin, HIGH);
            digitalWrite(ledGatePin, HIGH);
          }
          shuffleStep = 1;
          startCount = 0;
          sendStartTrigger = 1;
          clockCount = 0;
          barClockCount = 0;
          break;

        case midi::Stop:
          digitalWrite(dinGatePin, LOW);
          pause = 0;
          playMode = 0;
          if (mode == 0) {
            digitalWrite(outGatePin, LOW);
            digitalWrite(ledGatePin, LOW);
            sendClockTickOff = 1;
          }
          if (mode == 0) if (value[5] == 1 && playMode == 0) {
              sendClockTickOff = 1;
            }
          stopCount = 0;
          sendStopTrigger = 1;
          break;

        case midi::Clock:
          dinCount = 0;
          sendDinClock = 1;
          if (barClockCount == 0 && sendBeat1TriggerDone == 0) {
            beat1Count = 0;
            sendBeat1TriggerDone = 1;
            sendBeat1Trigger = 1;
          }
          // is this an offStep, which should be shuffled?



          if (clockCount == 0) {
            shuffleCount = 0;
            shuffleStep = !shuffleStep;
            if (mode == 0) {
              if (value[5] == 0) {
                sendClockTickOn = 1;
              }
              if (value[5] == 1 && playMode == 1 && pause <= 1) {
                sendClockTickOn = 1;
              }
            }
          }
          // count the loop cycles of one 16th note;
          // maxCycleCount will be used as tempodependent factor for the shuffle function
          if (cycleCount > 0 && clockStepCount == 0) {
            maxCycleCount = cycleCount;
            cycleCount = 0;
          }
          ++clockStepCount;
          if (clockStepCount == 24) clockStepCount = 0;
          // Next line for triggerpulses after the half clock time, only for dividers up to 1/16th



          if (clockCount == clockLength) {
            //if (debug == 1) Serial.println(3);
            if (mode == 0) {
              if (value[5] == 0) {
                sendClockTickOff = 1;
              }
              if (value[5] == 1 && playMode == 1 && pause <= 1) {
                sendClockTickOff = 1;
              }
            }
          }

          /*
            //  Next line for short trigger pulses and for clock dividers 32 / 48 / 96
            if ((value[1] == 1 || value[0] > 2) && clockCount == 1) {
            //if (debug == 1) Serial.println(4);
            if (mode == 0) {
              if (value[5] == 0) {
                sendClockTickOff = 1;
              }
              if (value[5] == 1 && playMode == 1 && pause == 0) {
                sendClockTickOff = 1;
              }
            }
            }
          */
          ++clockCount;
          ++ledBlink8th;
          ++barClockCount;
          if (value[3] == 0) barCount = 96;
          if (value[3] == 1) barCount = 192;
          if (value[3] == 2) barCount = 384;
          if (value[3] == 3) barCount = 72;
          if (value[3] == 4) barCount = 120;
          if (value[3] == 5) barCount = 84;
          if (barClockCount == barCount) barClockCount = 0;
          if (ledBlink8th > 5) ledBlink8th = 0;
          if (clockCount > ((clockLength * 2) - 1)) clockCount = 0;
          if ((value[2] == 2)
              && ((value[3] == 0 && barClockCount == 90) ||
                  (value[3] == 1 && barClockCount == 186) ||
                  (value[3] == 2 && barClockCount == 378) ||
                  (value[3] == 3 && barClockCount == 66) ||
                  (value[3] == 4 && barClockCount == 114) ||
                  (value[3] == 5 && barClockCount == 78))
             ) {
            resetCount = 0;
            sendResetTrigger = 1;
          }
          break;
        case midi::ControlChange:

          if (MIDI.getData1() == 70 && MIDI.getData2() <= maxvalue[0]) {
            value[0] = MIDI.getData2();
            //ledParam3(0);
            //delay(100);
            //ledValue(MIDI.getData2());
            EEPROM.write(0, value[0]);
            /*
              WE DO THIS NOW AT BEAT ONE
              if (value[0] == 0) clockLength = 3;
              if (value[0] == 1) clockLength = 6;
              if (value[0] == 2) clockLength = 12;
              if (value[0] == 3) clockLength = 4;
              if (value[0] == 4) clockLength = 2;
              if (value[0] == 5) clockLength = 1;
            */
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
            if (playMode == 1) digitalWrite(ledGatePin, HIGH);
          }
          if (MIDI.getData1() == 71 && MIDI.getData2() <= maxvalue[1]) {
            //ledParam3(1);
            //delay(100);
            //ledValue(MIDI.getData2());
            value[1] = MIDI.getData2();
            EEPROM.write(1, value[1]);
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
            if (playMode == 1) digitalWrite(ledGatePin, HIGH);
          }
          if (MIDI.getData1() == 72 && MIDI.getData2() <= maxvalue[2]) {
            //ledParam3(2);
            //delay(100);
            //ledValue(MIDI.getData2());
            value[2] = MIDI.getData2();
            EEPROM.write(2, value[2]);
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
            if (playMode == 1) digitalWrite(ledGatePin, HIGH);
          }
          if (MIDI.getData1() == 73 && MIDI.getData2() <= maxvalue[3]) {
            //ledParam3(3);
            //delay(100);
            //ledValue(MIDI.getData2());
            value[3] = MIDI.getData2();
            EEPROM.write(3, value[3]);
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
            if (playMode == 1) digitalWrite(ledGatePin, HIGH);
          }
          if (MIDI.getData1() == 74 && MIDI.getData2() <= maxvalue[4]) {
            //ledParam3(4);
            //delay(100);
            //ledValue(MIDI.getData2());
            value[4] = MIDI.getData2();
            EEPROM.write(4, value[4]);
            if ( value[4] <= 1 ) {
              pinMode(dinClockPin, INPUT);
              pinMode(dinGatePin, INPUT);
            }
            if ( value[4] >= 2 ) {
              pinMode(dinClockPin, OUTPUT);
              pinMode(dinGatePin, OUTPUT);
            }
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
            if (playMode == 1) digitalWrite(ledGatePin, HIGH);
          }
          if (MIDI.getData1() == 75 && MIDI.getData2() <= maxvalue[5]) {
            //ledParam3(5);
            //delay(100);
            //ledValue(MIDI.getData2());
            value[5] = MIDI.getData2();
            EEPROM.write(5, value[5]);
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
            if (playMode == 1) digitalWrite(ledGatePin, HIGH);
          }
          if (MIDI.getData1() == 76 && MIDI.getData2() <= maxvalue[6]) {
            control76 = MIDI.getData2();
          }


          break;
      }
    }



    // OLD DIN CLOCK - WITHOUT SHUFFLE
    if (sendDinClock == 1) {
      digitalWrite(dinClockPin, HIGH);
      ++dinCount;
      if (dinCount > 100) {
        digitalWrite(dinClockPin, LOW);
        dinCount = 0;
        sendDinClock = 0;
      }
    }
  }

  // */
  /* END OF MIDI IS MASTER */

  /* */
  //DIN-SYNC or VMC IS MASTER
  if (value[4] <= 1 || value[4] >= 4) {
    // Read the DIN-Socket
    if (value[4] <= 1) {
      dinGateState = digitalRead(dinGatePin);
      dinClockState = digitalRead(dinClockPin);
    }
    if (value[4] >= 4) {
      long vmcGateValue = analogRead(vmcGatePin);
      dinGateState = LOW;
      if (vmcGateValue > 512) {
        dinGateState = HIGH;
      }
      long vmcClockValue = analogRead(vmcClockPin);
      dinClockState = LOW;
      if (vmcClockValue > 512) {
        dinClockState = HIGH;
      }
    }

    //Create Start, Stop, and Reset Triggers from the incoming Controlsignal *
    if (dinGateState == 1 && dinGateStatePrev == 0) {
      MIDI.sendStart();
      playMode = 1;
      digitalWrite(outGatePin, HIGH);
      if (mode == 0 ) {
        digitalWrite(ledGatePin, HIGH);
      }
      shuffleStep = 1;
      startCount = 0;
      sendStartTrigger = 1;
      clockCount = 0;
      barClockCount = 0;
      dinGateStatePrev = 1;
    }
    if (dinGateState == 0 && dinGateStatePrev == 1) {
      MIDI.sendStop();
      playMode = 0;
      pause = 0;
      digitalWrite(outGatePin, LOW);
      if (value[5] == 1 && playMode == 0) {
        sendClockTickOff = 1;
      }

      if (mode == 0) {
        digitalWrite(ledGatePin, LOW);
      }
      stopCount = 0;
      sendStopTrigger = 1;
      dinGateStatePrev = 0;
    }

    // Create the Clock-Out from the incoming Clock
    if (dinClockState == 1 && dinClockStatePrev == 0) {
      MIDI.sendClock();
      if (barClockCount == 0 && sendBeat1TriggerDone == 0) {
        beat1Count = 0;
        sendBeat1TriggerDone = 1;
        sendBeat1Trigger = 1;
      }

      if (clockCount == 0) {
        shuffleCount = 0;
        shuffleStep = !shuffleStep;
        if (mode == 0) {
          if (value[5] == 0) {
            sendClockTickOn = 1;
          }
          //if (value[5] == 1 && playMode == 1 && pause <= 1) {
          if (value[5] == 1 && playMode == 1 && pause <= 1) {
            sendClockTickOn = 1;
          }
        }
      }

      // count the loop cycles of one 16th note;
      // maxCycleCount will be used as tempodependent factor for the shuffle function
      if (cycleCount > 0 && clockStepCount == 0) {
        maxCycleCount = cycleCount;
        cycleCount = 0;
      }
      ++clockStepCount;
      if (clockStepCount == 24) clockStepCount = 0;

      // Next line for triggerpulses after the half clock time, only for dividers up to 1/16th


      //if (clockCount == clockLength && value[0] < 3) {
      if (clockCount == clockLength) {
        if (mode == 0) {
          if (value[5] == 0) {
            sendClockTickOff = 1;
          }
          if (value[5] == 1 && playMode == 1 && pause <= 1) {
            sendClockTickOff = 1;
          }
        }
      }
      /*
        //  Next line for short trigger pulses and for clock dividers 32 / 48 / 96
        if (value[0] > 2 && clockCount == 1) {
        //if (debug == 1) Serial.println(4);
        if (mode == 0) {
          if (value[5] == 0) {
            sendClockTickOff = 1;
          }
          if (value[5] == 1 && playMode == 1 && pause == 0) {
            sendClockTickOff = 1;
          }
        }
        }
        //*/
      ++clockCount;
      ++barClockCount;
      ++ledBlink8th;
      if (ledBlink8th > 5) ledBlink8th = 0;
      if (value[3] == 0) barCount = 96;
      if (value[3] == 1) barCount = 192;
      if (value[3] == 2) barCount = 384;
      if (value[3] == 3) barCount = 72;
      if (value[3] == 4) barCount = 120;
      if (value[3] == 5) barCount = 84;
      if (barClockCount == barCount) barClockCount = 0;
      if (clockCount > ((clockLength * 2) - 1)) clockCount = 0;
      dinClockStatePrev = 1;
      /* */
      if ((value[2] == 2)

          && ((value[3] == 0 && barClockCount == 90) ||
              (value[3] == 1 && barClockCount == 184) ||
              (value[3] == 2 && barClockCount == 387) ||
              (value[3] == 3 && barClockCount == 66) ||
              (value[3] == 4 && barClockCount == 114) ||
              (value[3] == 5 && barClockCount == 78))

         ) {
        resetCount = 0;
        sendResetTrigger = 1;
      }
      // */
    }

    if (dinClockState == 0 && dinClockStatePrev == 1) {
      dinClockStatePrev = 0;
    }

    /* */
    if (MIDI.read()) {
      byte type = MIDI.getType();
      switch (type) {
        case midi::ControlChange:

          if (MIDI.getData1() == 70 && MIDI.getData2() <= maxvalue[0] && MIDI.getChannel() == channel) {
            value[0] = MIDI.getData2();
            //ledParam3(0);
            //delay(100);
            //ledValue(MIDI.getData2());
            EEPROM.write(0, value[0]);
            //setDivisor();
            /*
              if (value[0] == 0) clockLength = 3;
              if (value[0] == 1) clockLength = 6;
              if (value[0] == 2) clockLength = 12;
              if (value[0] == 3) clockLength = 3;
              if (value[0] == 4) clockLength = 2;
              if (value[0] == 5) clockLength = 1;
            */
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
          }
          if (MIDI.getData1() == 71 && MIDI.getData2() <= maxvalue[1] && MIDI.getChannel() == channel) {
            //ledParam3(1);
            //delay(100);
            //ledValue(MIDI.getData2());
            value[1] = MIDI.getData2();
            EEPROM.write(1, value[1]);
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
          }
          if (MIDI.getData1() == 72 && MIDI.getData2() <= maxvalue[2] && MIDI.getChannel() == channel) {
            //ledParam3(2);
            //delay(100);
            //ledValue(MIDI.getData2());
            value[2] = MIDI.getData2();
            EEPROM.write(2, value[2]);
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
          }
          if (MIDI.getData1() == 73 && MIDI.getData2() <= maxvalue[3] && MIDI.getChannel() == channel) {
            //ledParam3(3);
            //delay(100);
            //ledValue(MIDI.getData2());
            value[3] = MIDI.getData2();
            EEPROM.write(3, value[3]);
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
          }
          if (MIDI.getData1() == 74 && MIDI.getData2() <= maxvalue[4] && MIDI.getChannel() == channel) {
            //ledParam3(4);
            //delay(100);
            //ledValue(MIDI.getData2());
            value[4] = MIDI.getData2();
            if ( value[4] <= 1 ) {
              pinMode(dinClockPin, INPUT);
              pinMode(dinGatePin, INPUT);
            }
            if ( value[4] >= 2 ) {
              pinMode(dinClockPin, OUTPUT);
              pinMode(dinGatePin, OUTPUT);
            }
            if ( (value[4] == 2) || (value[4] == 3)) {
              MIDI.turnThruOn();
            } else {
              MIDI.turnThruOff();
            }

            EEPROM.write(4, value[4]);
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
          }
          if (MIDI.getData1() == 75 && MIDI.getData2() <= maxvalue[5] && MIDI.getChannel() == channel) {
            //ledParam3(5);
            delay(100);
            //ledValue(MIDI.getData2());
            value[5] = MIDI.getData2();
            EEPROM.write(5, value[5]);
            //blinkAll();
            ledblinkcount = 1;     // blink how many times
            ledblinklength = 5;   // defines the length of each blink
            ledpattern = "all";  // defines which Leds to use
            ledblinkcounter = 0;   // helper
            ledblinkdirection = 1; // helper
            ledblink = 1;          // enables blink
          }
          if (MIDI.getData1() == 76) {
            control76 = MIDI.getData2();
          }

          break;
      }
    }
    //*/
  }
  // */
  /* END OF DIN-SYNC IS MASTER */

  //cycleCount is counted to derive the loopcyles per beat
  cycleCount = cycleCount + 1;

  //maxCycleCount is a value derived from the incoming MIDI-Clocktempo
  if (value[0] < 3) {
    if (value[0] == 0) {
      mult = 1;
    }
    if (value[0] == 1) {
      mult = 2;
    }
    shuffleLength = (value[1] * (maxCycleCount / 70) * (value[0] + 1) * shuffleStep);
  }


  if (sendClockTickOn == 1) {
    if (shuffleCount >= shuffleLength) {
      sendClockTickOn = 0;
      clockTickCount = 0;
      //MIDI.sendNoteOn(1, 1, channel);
      digitalWrite(outClockPin, HIGH);
      digitalWrite(ledClockPin, HIGH);
    }
    ++shuffleCount;
  }

  if (sendClockTickOff == 1) {
    if (shuffleCount >= (shuffleLength / 2)) {
      sendClockTickOff = 0;
      clockTickCount = 0;
      digitalWrite(outClockPin, LOW);
      digitalWrite(ledClockPin, LOW);
    }
    ++shuffleCount;
    /*
      digitalWrite(outClockPin, LOW);
      digitalWrite(ledClockPin, LOW);
    */
  }

  if (sendBeat1Trigger == 1) {
    setDivisor();
    digitalWrite(outBeat1Pin, HIGH);
    if (mode == 0) {
      digitalWrite(ledBeat1Pin, HIGH);
    }
    ++beat1Count;
    if (beat1Count >= triggerLength) {
      digitalWrite(outBeat1Pin, LOW);
      if (mode == 0) {
        digitalWrite(ledBeat1Pin, LOW);
      }
      sendBeat1Trigger = 0;
      sendBeat1TriggerDone = 0;
    }
  }


  if (sendStartTrigger == 1) {
    digitalWrite(outStartPin, HIGH);
    digitalWrite(ledStartPin, HIGH);
    //A-155 mode
    if (value[2] == 1) {
      digitalWrite(outResetPin, HIGH);
      digitalWrite(ledResetPin, HIGH);
    }
    ++startCount;
    if (startCount >= triggerLength) {
      digitalWrite(outStartPin, LOW);
      digitalWrite(ledStartPin, LOW);
      //A-155 mode
      if (value[2] == 1) {
        digitalWrite(outResetPin, LOW);
        digitalWrite(ledResetPin, LOW);
      }
      sendStartTrigger = 0;
    }
  }

  if (sendStopTrigger == 1) {
    digitalWrite(outStopPin, HIGH);
    digitalWrite(ledStopPin, HIGH);
    ++stopCount;
    if (stopCount >= triggerLength) {
      digitalWrite(outStopPin, LOW);
      digitalWrite(ledStopPin, LOW);
      sendStopTrigger = 0;
      //A-154 mode
      if (value[2] == 0) {
        resetCount = 0;
        sendResetTrigger = 1;
      }
    }
  }

  if (sendResetTrigger == 1) {
    if (value[2] > 0) {
      digitalWrite(outResetPin, HIGH);
      digitalWrite(ledResetPin, HIGH);
    }
    ++resetCount;
    if (value[2] == 0 && resetCount >= triggerLength * 10) {
      digitalWrite(outResetPin, HIGH);
      digitalWrite(ledResetPin, HIGH);
    }
    if (value[2] > 0 && dinClockState == 0 && resetCount >= triggerLength) {
      digitalWrite(outResetPin, LOW);
      digitalWrite(ledResetPin, LOW);
      sendResetTrigger = 0;
    }

    if (value[2] == 0 && dinClockState == 0 && resetCount >= (triggerLength * 11)) {
      digitalWrite(outResetPin, LOW);
      digitalWrite(ledResetPin, LOW);
      sendResetTrigger = 0;
    }
  }





  /* Now we read the order of the button presses */

  /* Prepare the variables */
  if ((startStatePrev == 0 && stopStatePrev == 0) && (startState == 1 || stopState == 1))  {
    keyPressOrder = "";
    if (keyPressOrderLastIsSet == 1) {
      keyPressOrderLastIsSet = 0;
    }
  }
  /* Read button presses and fill the variable */
  if (startStatePrev == 0 && startState == 1) {
    keyPressOrder = keyPressOrder + "P";
    startStatePrev = 1;
  }
  if (startStatePrev == 1 && startState == 0) {
    keyPressOrder = keyPressOrder + "p";
    startStatePrev = 0;
  }
  if (stopStatePrev == 0 && stopState == 1) {
    keyPressOrder = keyPressOrder + "S";
    stopStatePrev = 1;
  }
  if (stopStatePrev == 1 && stopState == 0) {
    keyPressOrder = keyPressOrder + "s";
    stopStatePrev = 0;
  }
  /* make the variable readable */
  if (stopState == 0 && startState == 0 ) {
    if (keyPressOrderLastIsSet == 0) {
      keyPressOrderLast = keyPressOrder;
      keyPressOrderLastIsSet = 1;
    }
  }


  /* evaluating the order of button presses */

  /* Factory reset is in all modes possible: Hold STOP and Press START four times */
  if (keyPressOrderLast == "SPpPpPpPps") {
    factoryreset();
    keyPressOrderLast = "";
  }

  /* set MIDI-Channel by CC#76 */
  if (keyPressOrderLast == "PSsSsp") {

    setMidiChannel = 1;
    keyPressOrderLast = "";
    //blinkAll();
    ledblinkcount = 1000;     // blink how many times
    ledblinkcounter = 0;   // helper
    ledblinkdirection = 1; // helper
    ledblink = 1;          // enables blink
    ledblinklength = 20;   // defines the length of each blink
    ledpattern = "all";  // defines which Leds to use

  }

  if ( setMidiChannel == 1 ) {
    if (keyPressOrderLast == "Ss") {
      setMidiChannel = 0;
      keyPressOrderLast = "";
      //blinkAll();
      ledblinkcount = 3;     // blink how many times
      ledblinklength = 5;   // defines the length of each blink
      ledpattern = "all";  // defines which Leds to use
      ledblinkcounter = 0;   // helper
      ledblinkdirection = 1; // helper
      ledblink = 1;          // enables blink
    }
  }


  if ( setMidiChannel == 1 && control76 < 17 && control76 > 0) {
    //if ( control76 < 17 && control76 > 0) {

    value[6] = control76;
    EEPROM.write(6, value[6]);
    channel = control76;
    MIDI.setInputChannel(channel);

    control76 = 0;
    //setMidiChannel = 0;
    //blinkAll();
    ledblinkcount = 3;     // blink how many times
    ledblinkcounter = 0;   // helper
    ledblinkdirection = 1; // helper
    ledblink = 1;          // enables blink
    ledblinklength = 20;   // defines the length of each blink
    ledpattern = "all";  // defines which Leds to use

  }
  if ( setMidiChannel == 0 && control76 < 17 && control76 > 0) {
    control76 = 0;
  }







  /*
    if ( setMidiChannel == 1 ) {
      //blinkAll();
      if (MIDI.read()) {
        byte type = MIDI.getType();
        if (type == midi::ControlChange) {
          if (MIDI.getData1() == 76 && MIDI.getData2() <= 16 && MIDI.getChannel() == channel) {
            value[6] = MIDI.getData2();
            channel = value[6];
            setMidiChannel = 0;
          }
        }
      }
    }
  */

  /* PAUSE FUNCTION
    pause==0 -> normal play
    pause==1 -> wait for stop -> stop blinks gaeet = on
    pause==2 -> pause mode -> gate blinks
    pause==3 -> wait for play -> gate blinks gate = off
  */
  if (mode == 0) {
    if (keyPressOrderLast == "Ss" && pause == 0) {
      digitalWrite(ledStopPin, LOW);
      pause = 1; //waitForStop
      keyPressOrderLast = "";
    }

    if (pause == 1 && keyPressOrderLast == "Pp") {
      digitalWrite(ledStopPin, LOW);
      pause = 0;
      keyPressOrderLast = "";
    }



    if (pause == 1 && barClockCount == 0) {
      playMode = 0;
      pause = 2; //stopped
      digitalWrite(outGatePin, LOW);
      digitalWrite(ledGatePin, LOW);
      /*
        0=DIN, analog paused
        1=DIN, analog and MIDI paused
        2=DIN, analog, MIDI and DIN paused
        3=MIDI, analog paused
        4=MIDI, analog and DIN paused
        5=MIDI, analog, DIN and MIDI paused
      */
      if (value[4] == 1 || value[4] == 5) {
        MIDI.sendStop();
      }
      if (value[4] == 3 || value[4] == 5) {
        digitalWrite(dinGatePin, LOW);
      }
      if (value[5] == 1 && pause == 1) {
        digitalWrite(outClockPin, LOW);
        digitalWrite(ledClockPin, LOW);
      }
      stopCount = 0;
      sendStopTrigger = 1;
    }

    if (keyPressOrderLast == "Pp" && pause == 2) {
      //playMode = 1;
      digitalWrite(ledStopPin, LOW);
      pause = 3; //waitForPlay
      keyPressOrderLast = "";
    }

    /*cancels wait for play*/
    if (pause == 3 && keyPressOrderLast == "Ss") {
      digitalWrite(ledStopPin, LOW);
      pause = 2;
      keyPressOrderLast = "";
    }

    if (barClockCount == 1 && pause == 3) {
      playMode = 1;
      pause = 0;
      digitalWrite(outGatePin, HIGH);
      digitalWrite(ledGatePin, HIGH);
      if (value[4] == 1 || value[4] == 5) {
        MIDI.sendStart();
      }
      if (value[4] == 3 || value[4] == 5) {
        digitalWrite(dinGatePin, HIGH);
      }
      startCount = 0;
      sendStartTrigger = 1;
      digitalWrite(outClockPin, HIGH);
      digitalWrite(ledClockPin, HIGH);
      keyPressOrderLast = "";
    }

    if (ledBlink8th == 0) {
      if (pause == 1) {
        digitalWrite(ledStopPin, HIGH);
      }
      if (pause == 2) {
        digitalWrite(ledGatePin, HIGH);
      }
      if (pause == 3) {
        digitalWrite(ledStartPin, HIGH);
      }
    }
    if (ledBlink8th > 0) {
      if (pause == 1 ) {
        digitalWrite(ledStopPin, LOW);
      }
      if (pause == 2 ) {
        digitalWrite(ledGatePin, LOW);
      }
      if (pause == 3 ) {
        digitalWrite(ledStartPin, LOW);
      }
    }





    // Enter edit-parameter select mode sends STOP
    if (keyPressOrderLast == "SPsp") {
      stopCount = 0;
      sendStopTrigger = 1;
      allOff();
      digitalWrite(outClockPin, LOW);
      digitalWrite(ledClockPin, LOW);
      MIDI.sendStop();
      digitalWrite(dinGatePin, LOW);
      //blink3Left1();
      //delay(300);
      ledblinkcount = 2;     // blink how many times
      ledblinklength = 5;   // defines the length of each blink
      ledpattern = "left3";  // defines which Leds to use
      ledblinkcounter = 0;   // helper
      ledblinkdirection = 1; // helper
      ledblink = 1;          // enables blink
      //ledParam3(param);
      keyPressOrderLast = "";
      mode = 1;
    }
  }

  if (mode == 1) {
    if (param == 0)  digitalWrite(ledClockPin, HIGH);
    if (param == 1)  digitalWrite(ledStartPin, HIGH);
    if (param == 2)  digitalWrite(ledResetPin, HIGH);
    if (param == 3)  digitalWrite(ledBeat1Pin, HIGH);
    if (param == 4)  digitalWrite(ledStopPin, HIGH);
    if (param == 5)  digitalWrite(ledGatePin, HIGH);

    //exit edit-parameter mode
    if (keyPressOrderLast == "PSps") {
      mode = 0;
      allLedOff();
      sendCirclingLeds();
      keyPressOrderLast = "";
    }
    if (keyPressOrderLast == "Pp") {
      ++param;
      if (param == 6) param = 0;
      ledValue(value[param]);
      //allLedOff();
      keyPressOrderLast = "";
    }
    if (keyPressOrderLast == "Ss") {
      --param;
      if ((param + 1) == 0) param = 5;
      ledValue(value[param]);
      //allLedOff();
      keyPressOrderLast = "";
    }
    //enter edit-value mode
    if (keyPressOrderLast == "SPsp") {
      ledblinkcount = 2;     // blink how many times
      ledblinklength = 5;   // defines the length of each blink
      ledpattern = "right3";  // defines which Leds to use
      ledblinkcounter = 0;   // helper
      ledblinkdirection = 1; // helper
      ledblink = 1;          // enables blink
      keyPressOrderLast = "";
      mode = 2;
    }
  }

  if (mode == 2) {
    if (param == 0)  digitalWrite(ledClockPin, HIGH);
    if (param == 1)  digitalWrite(ledStartPin, HIGH);
    if (param == 2)  digitalWrite(ledResetPin, HIGH);
    if (param == 3)  digitalWrite(ledBeat1Pin, HIGH);
    if (param == 4)  digitalWrite(ledStopPin, HIGH);
    if (param == 5)  digitalWrite(ledGatePin, HIGH);
    //save value and exit edit-value mode
    if (keyPressOrderLast == "PSps") {
      EEPROM.write(param, value[param]);
      ledblinkcount = 3;     // blink how many times
      ledblinkcounter = 0;   // helper
      ledblinkdirection = 1; // helper
      ledblink = 1;          // enables blink
      ledblinklength = 5;   // defines the length of each blink
      ledpattern = "all";  // defines which Leds to use


      if (value[4] <= 1 ) {
        pinMode(dinClockPin, INPUT);
        pinMode(dinGatePin, INPUT);
      }
      if (value[4] >= 2 ) {
        pinMode(dinClockPin, OUTPUT);
        pinMode(dinGatePin, OUTPUT);
      }
      if ( (value[4] == 2) || (value[4] == 3)) {
        MIDI.turnThruOn();
      } else {
        MIDI.turnThruOff();
      }

      keyPressOrderLast = "";
      mode = 1;
    }
    if (keyPressOrderLast == "Pp") {
      ++value[param];
      if (value[param] > (maxvalue[param] )) value[param] = 0;
      ledValue(value[param]);
      keyPressOrderLast = "";
    }
    if (keyPressOrderLast == "Ss") {
      --value[param];
      if (value[param] < 0) value[param] = maxvalue[param];
      allLedOff();
      ledValue(value[param]);
      keyPressOrderLast = "";
    }
  }


  /* THIS IS THE BLINKENLIGHTS SEKTION*/

  if (ledblink > 0) {
    if (ledblinkdirection == 1) {
      ++ledblink;
      if ( ledpattern == "right3" || ledpattern == "all" || ledpattern == "lower2") digitalWrite(ledClockPin, HIGH);
      if ( ledpattern == "right3" || ledpattern == "all" || ledpattern == "middle2" || ledpattern == "start" || ledpattern == "startstop") digitalWrite(ledStartPin, HIGH);
      if ( ledpattern == "right3" || ledpattern == "all" || ledpattern == "upper2") digitalWrite(ledResetPin, HIGH);
      if ( ledpattern == "left3" || ledpattern == "all" || ledpattern == "lower2") digitalWrite(ledGatePin, HIGH);
      if ( ledpattern == "left3" || ledpattern == "all" || ledpattern == "middle2" || ledpattern == "stop" || ledpattern == "startstop") digitalWrite(ledStopPin, HIGH);
      if ( ledpattern == "left3" || ledpattern == "all" || ledpattern == "upper2") digitalWrite(ledBeat1Pin, HIGH);
    }
    if (ledblinkdirection == 0) {
      --ledblink;
      if ( ledpattern == "right3" || ledpattern == "all" || ledpattern == "lower2") digitalWrite(ledClockPin, LOW);
      if ( ledpattern == "right3" || ledpattern == "all" || ledpattern == "middle2" || ledpattern == "start" || ledpattern == "startstop") digitalWrite(ledStartPin, LOW);
      if ( ledpattern == "right3" || ledpattern == "all" || ledpattern == "upper2") digitalWrite(ledResetPin, LOW);
      if ( ledpattern == "left3" || ledpattern == "all" || ledpattern == "lower2") digitalWrite(ledGatePin, LOW);
      if ( ledpattern == "left3" || ledpattern == "all" || ledpattern == "middle2" || ledpattern == "stop" || ledpattern == "startstop") digitalWrite(ledStopPin, LOW);
      if ( ledpattern == "left3" || ledpattern == "all" || ledpattern == "upper2") digitalWrite(ledBeat1Pin, LOW);
    }
    if (ledblink > 100 * ledblinklength) {
      ledblinkdirection = 0;
    }
    if (ledblink == 1 && ledblinkdirection == 0 ) {
      if (ledblinkcounter < (ledblinkcount - 1)) {
        ledblinkcounter++;
        ledblinkdirection = 1;
      } else {
        ledblink = 0;
      }
    }
  }



}
/* END OF THE MAIN PROGRAM */

void setDivisor() {
  if (value[0] == 0) clockLength = 3;
  if (value[0] == 1) clockLength = 6;
  if (value[0] == 2) clockLength = 12;
  if (value[0] == 3) clockLength = 4;
  if (value[0] == 4) clockLength = 2;
  if (value[0] == 5) clockLength = 1;
}


void ledValue(int thisValue) {
  if (thisValue == 0)  digitalWrite(ledClockPin, HIGH);
  if (thisValue == 1)  digitalWrite(ledStartPin, HIGH);
  if (thisValue == 2)  digitalWrite(ledResetPin, HIGH);
  if (thisValue == 3)  digitalWrite(ledBeat1Pin, HIGH);
  if (thisValue == 4)  digitalWrite(ledStopPin, HIGH);
  if (thisValue == 5)  digitalWrite(ledGatePin, HIGH);
  delay(100);
  allLedOff();
  delay(100);
  if (thisValue == 0)  digitalWrite(ledClockPin, HIGH);
  if (thisValue == 1)  digitalWrite(ledStartPin, HIGH);
  if (thisValue == 2)  digitalWrite(ledResetPin, HIGH);
  if (thisValue == 3)  digitalWrite(ledBeat1Pin, HIGH);
  if (thisValue == 4)  digitalWrite(ledStopPin, HIGH);
  if (thisValue == 5)  digitalWrite(ledGatePin, HIGH);
  delay(100);
  allLedOff();

}




void sendResetPulse() {
  digitalWrite(outResetPin, HIGH);
  digitalWrite(ledResetPin, HIGH);
  for (resetCount = 0; resetCount <= triggerLength; ++resetCount) {
    digitalWrite(outResetPin, HIGH);
    digitalWrite(ledResetPin, HIGH);
  }
  digitalWrite(outResetPin, LOW);
  digitalWrite(ledResetPin, LOW);
}
/*
  void sendBeat1Pulse() {
  digitalWrite(outBeat1Pin, HIGH);
  for (beat1Count = 0; beat1Count <= triggerLength; ++beat1Count) {
    digitalWrite(outBeat1Pin, HIGH);
  }
  digitalWrite(outBeat1Pin, LOW);
  }
*/
void allOff() {
  digitalWrite(outClockPin, LOW);
  digitalWrite(outStartPin , LOW);
  digitalWrite(outResetPin, LOW);
  digitalWrite(outBeat1Pin, LOW);
  digitalWrite(outStopPin, LOW);
  digitalWrite(outGatePin, LOW);
  allLedOff();
}

void allLedOff() {
  digitalWrite(ledClockPin, LOW);
  digitalWrite(ledStartPin , LOW);
  digitalWrite(ledResetPin, LOW);
  digitalWrite(ledBeat1Pin, LOW);
  digitalWrite(ledStopPin, LOW);
  digitalWrite(ledGatePin, LOW);
}

void sendCirclingLeds() {
  for (int i = 0; i < 3; ++i) {
    digitalWrite(ledClockPin, HIGH);
    delay(50);
    digitalWrite(ledClockPin, LOW);
    digitalWrite(ledStartPin , HIGH);
    delay(50);
    digitalWrite(ledStartPin , LOW);
    digitalWrite(ledResetPin, HIGH);
    delay(50);
    digitalWrite(ledResetPin, LOW);
    digitalWrite(ledBeat1Pin, HIGH);
    delay(50);
    digitalWrite(ledBeat1Pin, LOW);
    digitalWrite(ledStopPin, HIGH);
    delay(50);
    digitalWrite(ledStopPin, LOW);
    digitalWrite(ledGatePin, HIGH);
    delay(50);
    digitalWrite(ledGatePin, LOW);
  }
  //digitalWrite(outStopPin, HIGH);
  //for (stopCount = 0; stopCount <= (triggerLength * 20); ++stopCount) {
  //  digitalWrite(outStopPin, HIGH);
  //}
  //digitalWrite(outStopPin, LOW);

  //sendResetPulse();
}

void factoryreset() {
  EEPROM.write(0, 2); //div 1/16th
  EEPROM.write(1, 0); //no shuffle
  EEPROM.write(2, 0); //reset after stop
  EEPROM.write(3, 0); //beat1 1Bar
  EEPROM.write(4, 5); //DIN 1 or 2, MIDI 2 or 3, VMC 4 or 5
  EEPROM.write(5, 0); //Continous Clock
  EEPROM.write(6, 4); //MIDI-Channel 1
  delay(100);
  value[0] = EEPROM.read(0);
  value[2] = EEPROM.read(1);
  value[1] = EEPROM.read(2);
  value[3] = EEPROM.read(3);
  value[4] = EEPROM.read(4);
  value[5] = EEPROM.read(5);
  value[6] = EEPROM.read(6);
  mode = 0;
  channel = value[6];
  MIDI.setInputChannel(channel);
  setMidiChannel = 0;
  //clockLength = 3;
  sendCirclingLeds();
}
